/*
  Copyright 2004 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "tuneparams.h"
#include "dvbtuneparamscommon.h"
#include "stringutil.h"

#include <map>
#include <iostream>
#include <cstdlib>
#include "tune.h"

// DVB-T Tuning Parameters 

DVBCTuneParams::DVBCTuneParams(const std::string &name, int apid, int vpid)
  : TuneParams(name, apid, vpid),
    freq(0),
    inv(INVERSION_OFF),
    sr(0),
    fec(HP_CODERATE_DEFAULT)
{
}

DVBCTuneParams::~DVBCTuneParams()
{
}

DVBCTuneParams *DVBCTuneParams::createFromCZAP(const std::string &line)
{
  // It is assumed that the line is a valid CZAP conf line
  // (not a comment) 

  std::vector<std::string> v;
  splitString(line, ':', v);

  std::string &newName = v[0];
  int vpid = toUnsignedLong(v[6]);
  int apid = toUnsignedLong(v[7]);

  DVBCTuneParams *p = new DVBCTuneParams(newName, vpid, apid);
  p->freq = toUnsignedLong(v[1]);
  p->inv = translateTuneParam(v[2]);
  p->sr = toUnsignedLong(v[3]);
  p->fec = translateTuneParam(v[4]);
  p->qam = translateTuneParam(v[5]);
  return p;
}

bool DVBCTuneParams::tune(int frontendFD) const
{
  // Possible todo:
  // Change tune_it to observe lower code rate (fec2)

  // Give it three goes at least!
  for (int i = 0; i < 3; i++) {
    int rc = tune_it(frontendFD, 
		     freq, 
		     sr,       	// Symbol Rate for DVB-C or DVB-S 
		     0,		// Polarity 
		     -1,	// Tone  
		     fe_spectral_inversion_t(inv), 
		     0,		// diseqc 
		     fe_modulation_t(qam),
		     fe_code_rate_t(fec),
		     TRANSMISSION_MODE_DEFAULT,
		     GUARD_INTERVAL_DEFAULT,
		     BANDWIDTH_DEFAULT,
		     LP_CODERATE_DEFAULT,
		     HIERARCHY_DEFAULT);

    if (rc >= 0)
      return true;
  }

  std::cerr << "Fatal error: failed to tune channel " << name << "\n";
  return false;
}

bool DVBCTuneParams::sameAsExceptPIDs(const TuneParams &other) const
{
  const DVBCTuneParams &x = dynamic_cast<const DVBCTuneParams&>(other);

  if (&x == NULL)
    return false;
 
  return (freq == x.freq &&
	  inv == x.inv &&
	  sr == x.sr &&
	  fec == x.fec &&
	  qam == x.qam);
}
